==> nodus-framework-0.0.22.tgz: package/lib/actions.js <==
'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const Promise = require('bluebird');
const functions = require('./functions');
const microtime = require('microtime');
const EventEmitter = require('./events').EventEmitter;

class Action extends EventEmitter {
    constructor(name, handler, options) {
        super();

        options = options || {};

        this._name = name;
        this._handler = handler;
        this._options = options;
    }

    get name() {
        return this._name;
    }

    get handler() {
        return this._handler;
    }

    get run() {
        // Action will invoke func and add some tracing
        const self = this;
        return function () {

            // Track when the action was started
            const started_on = microtime.now();

            // Function that runs when the activity is completed
            const complete = (result, error) => {

                // Track when the action completed
                const completed_on = microtime.now();
                const elapsed_ms = (completed_on - started_on) * .001;
                const success = util.isNullOrUndefined(error);

                const trace = {
                    success: success,
                    started_on: started_on,
                    completed_on: completed_on,
                    elapsed_ms: elapsed_ms,
                    arguments: functions.Arguments(arguments)
                };

                if (success)
                    self.emit('result', result, trace);
                else
                    self.emit('error', error, trace);

                self.emit('completed', trace);
            };

            return Promise
                .try(() => self.handler.apply(self, arguments)) // KEEP THIS CONTEXT OF THE FUNCTION
                .tap(result => complete(result))
                .tapCatch(error => complete(null, error))
        };
    }
}

/**
 *
 * @param func
 * @param options
 * @param cb
 */
function createAction(name, handler, options) {

    // createAction(func) -> createAction(func.name, func)
    if (arguments.length === 1 && util.isFunction(arguments[0]))
        return createAction(arguments[0].name, arguments[0]);

    const action = new Action(name, handler, options);

    // const ret = function () {
    //     // Run the action
    //     return action.run.apply(this, arguments);
    // };
    //
    // // Return the action attached to the function
    // ret.action = action;

    // Return the action itself
    return action;
}

// ** Exports
module.exports = createAction;
module.exports.Action = Action;

==> nodus-framework-0.0.28.tgz: package/lib/actions.js <==
'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const Promise = require('bluebird');
const functions = require('./functions');
const microtime = require('microtime');
const EventEmitter = require('./events').EventEmitter;
const logger = require('./logging').createLogger('actions');

class Action extends EventEmitter {
    constructor(name, handler, options) {
        super();

        options = options || {};

        this._name = name;
        this._handler = handler;
        this._options = options;
    }

    get name() {
        return this._name;
    }

    get handler() {
        return this._handler;
    }

    get run() {
        // Action will invoke func and add some tracing
        const self = this;
        return function () {
            // logger.debug('ACTION:THIS', this);

            // Track when the action was started
            const started_on = microtime.now();

            // Function that runs when the activity is completed
            const complete = (result, error) => {

                // Track when the action completed
                const completed_on = microtime.now();
                const elapsed_ms = (completed_on - started_on) * .001;
                const success = util.isNullOrUndefined(error);

                const trace = {
                    success: success,
                    started_on: started_on,
                    completed_on: completed_on,
                    elapsed_ms: elapsed_ms,
                    arguments: functions.Arguments(arguments)
                };

                if (success)
                    self.emit('result', result, trace);
                else
                    self.emit('error', error, trace);

                self.emit('completed', trace);
            };

            return Promise
                .try(() => self.handler.apply(this, arguments)) // KEEP THIS CONTEXT OF THE FUNCTION
                .tap(result => complete(result))
                .tapCatch(error => complete(null, error))
        };
    }
}

/**
 *
 * @param func
 * @param options
 * @param cb
 */
function createAction(name, handler, options) {

    // createAction(func) -> createAction(func.name, func)
    if (arguments.length === 1 && util.isFunction(arguments[0]))
        return createAction(arguments[0].name, arguments[0]);

    // createAction(name, options) -> createAction(name, options.handler, options)
    if (arguments.length === 2 && util.isObject(arguments[1]))
        return createAction(arguments[0], arguments[1].handler, arguments[1]);

    const action = new Action(name, handler, options);

    // const ret = function () {
    //     // Run the action
    //     return action.run.apply(this, arguments);
    // };
    //
    // // Return the action attached to the function
    // ret.action = action;

    // Return the action itself
    return action;
}

// ** Exports
module.exports = createAction;
module.exports.Action = Action;
